import { Component } from '@angular/core';
import {DatePipe, NgForOf, NgIf} from "@angular/common";
import {DictionaryPipe} from "../../../pipes/common/dictionary.pipe";
import {FormGroup, FormsModule, NonNullableFormBuilder, ReactiveFormsModule, Validators} from "@angular/forms";
import {NzButtonComponent} from "ng-zorro-antd/button";
import {
    NzCellFixedDirective,
    NzTableCellDirective,
    NzTableComponent,
    NzTbodyComponent, NzTheadComponent,
    NzThMeasureDirective, NzTrDirective
} from "ng-zorro-antd/table";
import {NzColDirective, NzRowDirective} from "ng-zorro-antd/grid";
import {NzDatePickerComponent} from "ng-zorro-antd/date-picker";
import {NzDividerComponent} from "ng-zorro-antd/divider";
import {NzDropDownADirective, NzDropDownDirective, NzDropdownMenuComponent} from "ng-zorro-antd/dropdown";
import {NzFormControlComponent, NzFormDirective, NzFormItemComponent, NzFormLabelComponent} from "ng-zorro-antd/form";
import {NzIconDirective} from "ng-zorro-antd/icon";
import {NzImageDirective, NzImageGroupComponent, NzImageModule} from "ng-zorro-antd/image";
import {NzInputDirective} from "ng-zorro-antd/input";
import {
    NzInputNumberComponent,
    NzInputNumberGroupComponent,
    NzInputNumberGroupWhitSuffixOrPrefixDirective
} from "ng-zorro-antd/input-number";
import {NzMenuDirective, NzMenuItemComponent} from "ng-zorro-antd/menu";
import {NzModalComponent, NzModalContentDirective, NzModalFooterDirective, NzModalModule} from "ng-zorro-antd/modal";
import {NzOptionComponent, NzSelectComponent} from "ng-zorro-antd/select";
import {NzPopconfirmDirective} from "ng-zorro-antd/popconfirm";
import {NzUploadComponent} from "ng-zorro-antd/upload";
import {environment} from "../../../../environments/environment";
import {NzMessageService} from "ng-zorro-antd/message";
import {DiscountService} from "../../../services/marketing/discount.service";
import { fallbackImg } from '../../../data/goods/goods.namespace';
import {AuditService} from "../../../services/audit/audit.service";
import {NzTreeSelectComponent} from "ng-zorro-antd/tree-select";
import {NzDescriptionsComponent, NzDescriptionsItemComponent} from "ng-zorro-antd/descriptions";
import {NzBadgeComponent} from "ng-zorro-antd/badge";
import {AuditStatusPipe} from "../../../pipes/audit/audit-status.pipe";
import {NzTypographyComponent} from "ng-zorro-antd/typography";

@Component({
  selector: 'app-audit-list',
  standalone: true,
  imports: [
    DatePipe,
    DictionaryPipe,
    FormsModule,
    NgForOf,
    NgIf,
    NzButtonComponent,
    NzCellFixedDirective,
    NzColDirective,
    NzDatePickerComponent,
    NzDividerComponent,
    NzDropDownADirective,
    NzDropDownDirective,
    NzDropdownMenuComponent,
    NzFormControlComponent,
    NzFormDirective,
    NzFormItemComponent,
    NzFormLabelComponent,
    NzIconDirective,
    NzImageDirective,
    NzInputDirective,
    NzInputNumberComponent,
    NzInputNumberGroupComponent,
    NzInputNumberGroupWhitSuffixOrPrefixDirective,
    NzMenuDirective,
    NzMenuItemComponent,
    NzModalComponent,
    NzOptionComponent,
    NzPopconfirmDirective,
    NzRowDirective,
    NzModalModule,
    NzImageModule,
    NzSelectComponent,
    NzTableCellDirective,
    NzTableComponent,
    NzTbodyComponent,
    NzThMeasureDirective,
    NzTheadComponent,
    NzTrDirective,
    NzUploadComponent,
    ReactiveFormsModule,
    NzTreeSelectComponent,
    NzDescriptionsItemComponent,
    NzBadgeComponent,
    NzDescriptionsComponent,
    NzModalFooterDirective,
    NzModalContentDirective,
    AuditStatusPipe,
    NzImageGroupComponent,
    NzTypographyComponent
  ],
  templateUrl: './audit-list.component.html',
  styleUrl: './audit-list.component.less'
})
export class AuditListComponent {
  // 表单页数
  tablePageNum = 1;
  // 表单数据
  tableData: any = {
    total: 0,
    loading: false,
    list: [],
  };
  // 搜索表单
  searchForm: FormGroup;
  auditDetail: any = {
    goodsModel: {}
  };

  baseUrl = environment.baseUrl;
  imageUrl = environment.imageUrl;
  isVisibleDetail = false;
  isVisibleAudit = false;
  postAudit:any;
  remarks: string | undefined;

  protected readonly fallbackImg = fallbackImg;

  constructor(private fb: NonNullableFormBuilder,
              private msg: NzMessageService,
              private auditService: AuditService,
              private discountService: DiscountService) {

    // 初始化搜索框
    this.searchForm = this.fb.group({
      submitObjectName: [''],
      submitOpName: [''],
      status: [''],
    });

    this.getRequest();


  }

  showDetail(id: number): void {
    this.isVisibleDetail = true;
    let params =  {
      id: id,
      time: new Date().getTime()
    }
    this.auditService.auditDetail(params , (data: any) => {
      if (data['return_code'] == '000000') {
        this.auditDetail = data['return_data'];
      }
    })
  }

  // 查询列表
  public getRequest(reset: boolean = false) {
    this.tableData.loading  = true;
    if (reset) {
      this.tablePageNum = 1;
    }
    this.searchForm.value.pageNum = this.tablePageNum;
    this.searchForm.value.pageSize = 10;
    this.searchForm.value.time = new Date().getTime();

    this.auditService.getListGoodsAudit(this.searchForm.value , (data: any) => {
      if (data['return_code'] == '000000') {
        this.tableData = data['return_data'];
      } else {
        this.msg.error(data['return_msg']);
      }
      this.tableData.loading  = false;
    });
  }

  passOrOverrule(id: number , auditStatus: number): void {
    this.postAudit =  {
      id: id,
      time: new Date().getTime(),
      status: auditStatus
    }

    this.isVisibleAudit = true;

  }

  postAuditMsg(): void {
    this.postAudit.remarks = this.remarks;
    this.auditService.auditMsg(this.postAudit , (data: any) => {
      if (data['return_code'] == '000000') {
        this.getRequest(true);
        this.msg.success("操作成功")
      } else {
        this.msg.error(data['return_msg']);
      }
      this.isVisibleAudit  = false;
    });
  }

}
